'use strict';

/*
 * Lambda's Context object.
*/
var utils = require('./utils.js');

var logger, unmute;

/*
 * doneStatus & postDone were minimum; probably defined internally in Lambda.
 */
var doneStatus = false;
var postDone = function(error, message) {};

/*
 * Used to determine the getRemainingTimeInMillis()
 */
var startTime;
var timeout;

/*
 * Parse JSON data 
*/
var _parseJSON = function(userdata){
	var result = {};
	try{
		result = JSON.parse(userdata);
		return result;
	}catch(err){
		return result;
	}
}

function Context() {}

/*
 * exports
 */
module.exports = Context;

/*
 * Context object properties
 */
Context.callbackWaitsForEmptyEventLoop = true;
Context.invokedFunctionArn = 'a';
Context.memoryLimitInMB = 1;
Context.identity = null;

Context.requestId = '';
Context.accessKey = '';
Context.secretKey = '';
Context.authToken = '';
Context.securityToken = '';

Context.projectId = '';
Context.package = '';
Context.functionName = '';
Context.functionVersion = '';
Context.memory = '';
Context.cpu = '';
Context.userData = {};
Context.timeout = '3000';



/*
 * callback function called after done
 */
Context.callback = function(result) {
    logger.info("RES:" + result); 
    console.info("RES2:" + result); 
    return result;
};

/*
 * create random invokeid.
 * Assuming that invokeid follows the format:
 * 8hex-4hex-4hex-4hex-12hex
 */
Context.createInvokeId = function() {

    return [
        utils.generateRandomHex(8),
        utils.generateRandomHex(4),
        utils.generateRandomHex(4),
        utils.generateRandomHex(4),
        utils.generateRandomHex(12)
    ].join('-');
};

/*
 * Context initialization.
 * Called from lambda-run
 */
Context._initialize = function(options) {
    /* set time */
    startTime = new Date().getTime();
    timeout = options.timeoutMs;
    
    logger = options.logger;
    unmute = options.unmute;

    Context.timeout = options.timeoutMs ||  '4000';
    Context.projectId = process.env.RUNTIME_PROJECT_ID ||  '';
    Context.package = process.env.RUNTIME_PACKAGE ||  '';
    Context.functionName = process.env.RUNTIME_FUNC_NAME ||  '';
    Context.functionVersion = process.env.RUNTIME_FUNC_VERSION ||  '';
    Context.memory = process.env.RUNTIME_MEMORY ||  '0';
    Context.cpu = process.env.RUNTIME_CPU ||  '0';
    Context.userData = _parseJSON(process.env.RUNTIME_USERDATA) ||  '';
    Context.package = process.env.RUNTIME_PACKAGE ||  '';

    
    Context.requestId = options.requestId;
    Context.accessKey = options.accessKey;
    Context.secretKey = options.secretKey;
    Context.authToken = options.authToken;
    Context.securityToken = options.securityToken;
    /* Set callbackWaitsForEmptyEventLoop */
    Context.callbackWaitsForEmptyEventLoop = options.callbackWaitsForEmptyEventLoop;
    return;
};

/*
 * This `done` method is directly extracted from source.
 */
Context.done = function(err, message) {
    if(unmute != null) {unmute(); unmute = null;}
    if (!Context.callbackWaitsForEmptyEventLoop) {
        Context.callback(err, message);
    }

// /*
    if (err !== null) {
        console.info(err);
//        utils.outputJSON(err,logger);
    } else {
        console.info(message);
//        utils.outputJSON(message, logger);
    }
// */    
    if (Context.callbackWaitsForEmptyEventLoop) {
        Context.callback(err, message);
    }
};


/*
 * `fail` method calls the `done` method
 */
Context.fail = function(err) {
    if(unmute != null) {unmute(); unmute = null;}
    logger.log('error', 'FAILING!!');
    Context.done(err);
};

/*
 * `succeed` method calls the `done` method
 */
Context.succeed = function(message) {
    if(unmute != null) {unmute(); unmute = null;}
    Context.done(null, message);
};



//Gets the request ID associated with the request.
Context.getRequestID = function(){
    return Context.requestId;
};

Context.getProjectID = function(){
    return Context.projectId;
};

Context.getPackage = function(){
    return Context.package;
};

Context.getFunctionName = function(){
    return Context.functionName;
};

Context.getVersion = function(){
    return Context.functionVersion;
};

Context.getMemorySize = function(){
    return Context.memory;
};

Context.getCPUNumber = function(){
    return Context.cpu;
};


Context.getAccessKey = function(){
    return Context.accessKey;
}; 

Context.getSecretKey = function(){
    return Context.secretKey;
};


Context.getToken = function(){
    return Context.authToken;
};

Context.getUserData = function(key){
	var data = Context.userData[key];
    return data;
};

Context.getRunningTimeInSeconds = function() {
    return Context.timeout;
};

Context.getLogger = function() {
    return logger;
};

/*
 * 'getRemainingTimeInMillis' method return time before task is killed
 */
Context.getRemainingTimeInMillis = function() {
    var now = new Date().getTime();
    return (timeout + startTime - now);
};

Context.getSecurityToken = function(){
    return Context.securityToken;
};

