'use strict';

var utils = require('./utils.js'),
    _context = require('./context.js');


var logger = require('winston');

var _setLogger = function(_logger){
    if(_logger != null && typeof _logger.loggers != 'undefined'){
       logger = _logger;
    } else {
      console.warn("Invalid logger object ! Using default logger");
    }
}

var _getLogger = function() {
    return logger;
}

var _execute = function(opts) {
    var temp = process.env.RUNTIME_HANDLER || 'index.handler';
    var handler = temp.split('.');
    var fssPath = handler[0];
    var fssHandler = handler[1];
    var ak = opts.ak
    var sk = opts.sk
    var authToken = opts.authToken
    var securityToken = opts.securityToken

    var callback = opts.callback;
    var event = opts.event;
    var callbackWaitsForEmptyEventLoop = opts.callbackWaitsForEmptyEventLoop || true;
    var timeoutMs = opts.timeoutMs || 3000;

    var envBag = {
        fssPath: handler[0],
        fssHandler: handler[1],
        maxTimeout: (process.env.RUNTIME_TIMEOUT || 3) * 1000,
        region: 'unknown'
    };

    var obj = JSON.parse(process.env.RUNTIME_USERDATA);
    var keys = Object.keys(obj);
    for (var i = 0; i < keys.length; i++) {
        process.env[keys[i]]= obj[keys[i]];
    }

    var err;

    // load event & context
    var context = this.context;

    var loggerForUser = require('winston');

    const consoleTransport= new (loggerForUser.transports.Console)({
        handleExceptions:true,
        json:false,
        showLevel:false,
        prettyPrint: true,
        colorize:false,
        stringify:true,
        timestamp: function(){
            return new Date().toISOString();
        },
        formatter: function(options) {
            return options.timestamp() + ' ' + ' ' + options.message;
        }
    });
    loggerForUser.configure({
        transports: [
           consoleTransport
         ]
    });

    context._initialize({
        // functionName: fssHandler,
        invokeId: context.createInvokeId(), 
        timeoutMs: timeoutMs,
        callbackWaitsForEmptyEventLoop: callbackWaitsForEmptyEventLoop,
        logger: loggerForUser,
        accessKey: ak,
        secretKey: sk,
        authToken: authToken,
        securityToken: securityToken
    });

    // export the LAMBDA_TASK_ROOT enviroment variable
    process.env['LAMBDA_TASK_ROOT'] = process.cwd();

    //setting common other vars environments
    process.env['NODE_PATH'] = utils.getAbsolutePath('node_modules');
    process.env['SESSION_TOKEN'] = context.requestId;


    if (callback) context.callback = callback;

    var fssfuncPath=utils.getAbsolutePath(fssPath)
    var fssFunc 
    var errMessage = 'Results: '
    try{
        fssFunc = require(fssfuncPath);
    }catch(error){
        errMessage = errMessage + error
        console.log(errMessage);
        if (errMessage.indexOf(fssfuncPath) >= 0){
            callback("Not found module " + "'" + fssPath + "'" , null)
        } else {
            logger.info(errMessage + error);
            callback(errMessage, null)
        }
        return 
    }  

    if (fssFunc[fssHandler]){
        fssFunc[fssHandler](event, context, context.done);
    } else{
        err = 'Handler ' + "'" + fssHandler + "'" + ' not found in module ' + "'" + fssPath + "'";
        logger.info(err);
        callback(err,null);
    }
};

module.exports = {
    context: _context,
    execute: _execute, 
    setLogger: _setLogger,
    getLogger: _getLogger
};
