#!/bin/sh

USER=$(whoami)
OPERIP=$(whoami | awk '{print $NF}' | sed 's/[()]//g')
RuntimeUserEnvPrefix=_FSS_USER_ENV_

umask 027

check_run_user() {
    CUR_PATH=$(cd `dirname $0`;pwd)
    SCRIPT_PATH=$0
    SCRIPT_EXEC_USER="funcuser"
    CURRENT_USER="`/usr/bin/id -u -n`"
    if [ "${SCRIPT_EXEC_USER}" != "${CURRENT_USER}" ]
    then
        echo "only ${SCRIPT_EXEC_USER} can execute this script."
        exit 1
    fi
}

#check_run_user

RUNTIME_LOG_DIR=/var/log
if [[ -z ${RUNTIME_LOG_DIR} ]];then
     /bin/echo "RUNTIME_LOG_DIR is empty."
     exit 1
fi

LOG_FILE="${RUNTIME_LOG_DIR}/start_runtime.log"
if [ ! -e "$LOG_FILE" ];then
    touch "$LOG_FILE"
fi

#09-13 16:40:16,757 WARN (script.sh|pid)
LOG_TITLE="$(basename $0)|$$"
PRINT_LOG()
{
        LOG_LEVEL=$1
        LOG_INFO=$2

        TIME_INFO="`date +%m-%d` `date +%H:%M:%S`"
        if [ "${LOG_LEVEL}" != "INFO" -a "${LOG_LEVEL}" != "WARN" -a "${LOG_LEVEL}" != "FATAL" ]
        then
                LOG_LEVEL="WARN"
        fi

        echo "${TIME_INFO} ${LOG_LEVEL} (${LOG_TITLE}) ${LOG_INFO}" >> "${LOG_FILE}"

        if [ "${LOG_LEVEL}" = "FATAL" ]
        then
                exit 1;
        fi
}

PRINT_LOG "INFO" "start_runtime started."

if [ -z ${RUNTIME_ROOT} ]; then
   PRINT_LOG "FATAL"  "RUNTIME_ROOT is null"
   exit 1
fi

if [ -z ${RUNTIME_CODE_ROOT} ]; then
   PRINT_LOG "FATAL" "RUNTIME_CODE_ROOT is null"
   exit 1
fi

if [ -z ${RUNTIME_HANDLER} ]; then
   PRINT_LOG "INFO" "RUNTIME_HANDLER is null"
   export RUNTIME_HANDLER="index.handler"
fi

if [ -z ${RUNTIME_TIMEOUT} ]; then
   PRINT_LOG "INFO" "RUNTIME_TIMEOUT is null"
   export RUNTIME_TIMEOUT=3
fi

if [ -z ${DEBUG_PORT} ];then
    echo "No debug"
else
    DEBUG_OPT="--debug --debug-brk --debug-port=${DEBUG_PORT}"
    echo "DEBUG_OPT ${DEBUG_OPT}"
fi

export LD_LIBRARY_PATH=/usr/lib:${RUNTIME_CODE_ROOT}:$LD_LIBRARY_PATH

CUR_PATH=$(cd `dirname $0`;pwd)
PRINT_LOG "INFO" "will run nodeJS function."
logger -t $USER -p local0.info "$(basename $0);Successful;${OPERIP:-127.0.0.1};Start nodejs6.10 runtime."
cd ${RUNTIME_CODE_ROOT}

uuid=`cat /proc/sys/kernel/random/uuid`

${RUNTIME_ROOT}/rtsp/nodejs/bin/node ${DEBUG_OPT} ${RUNTIME_ROOT}/bin/runtime/bin/fss-run $uuid  "$1"

result=$?
exit $result

